@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2022 PgPool Global Development Group
+ * Copyright (c) 2003-2019	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -178,6 +178,52 @@ typedef struct
  */
 typedef struct HbaLine HbaLine;
 
+
+#ifdef USE_SSL
+/*
+ *	Hardcoded DH parameters, used in ephemeral DH keying.
+ *
+ *	If you want to create your own hardcoded DH parameters
+ *	for fun and profit, review "Assigned Number for SKIP
+ *	Protocols" (http://www.skip-vpn.org/spec/numbers.html)
+ *	for suggestions.
+ */
+#define FILE_DH2048 \
+"-----BEGIN DH PARAMETERS-----\n\
+MIIBCAKCAQEA9kJXtwh/CBdyorrWqULzBej5UxE5T7bxbrlLOCDaAadWoxTpj0BV\n\
+89AHxstDqZSt90xkhkn4DIO9ZekX1KHTUPj1WV/cdlJPPT2N286Z4VeSWc39uK50\n\
+T8X8dryDxUcwYc58yWb/Ffm7/ZFexwGq01uejaClcjrUGvC/RgBYK+X0iP1YTknb\n\
+zSC0neSRBzZrM2w4DUUdD3yIsxx8Wy2O9vPJI8BD8KVbGI2Ou1WMuF040zT9fBdX\n\
+Q6MdGGzeMyEstSr/POGxKUAYEY18hKcKctaGxAMZyAcpesqVDNmWn6vQClCbAkbT\n\
+CD1mpF1Bn5x8vYlLIhkmuquiXsNV6TILOwIBAg==\n\
+-----END DH PARAMETERS-----\n"
+#endif
+
+/*
+ * Macro that allows to cast constness away from an expression, but doesn't
+ * allow changing the underlying type.  Enforcement of the latter
+ * currently only works for gcc like compilers.
+ *
+ * Please note IT IS NOT SAFE to cast constness away if the result will ever
+ * be modified (it would be undefined behaviour). Doing so anyway can cause
+ * compiler misoptimizations or runtime crashes (modifying readonly memory).
+ * It is only safe to use when the the result will not be modified, but API
+ * design or language restrictions prevent you from declaring that
+ * (e.g. because a function returns both const and non-const variables).
+ *
+ * Note that this only works in function scope, not for global variables (it'd
+ * be nice, but not trivial, to improve that).
+ */
+#if defined(HAVE__BUILTIN_TYPES_COMPATIBLE_P)
+#define unconstify(underlying_type, expr) \
+	(StaticAssertExpr(__builtin_types_compatible_p(__typeof(expr), const underlying_type), \
+					  "wrong cast"), \
+	  (underlying_type) (expr))
+#else
+#define unconstify(underlying_type, expr) \
+	((underlying_type) (expr))
+#endif
+
 /*
  * stream connection structure
  */
@@ -417,7 +463,7 @@ typedef enum
 #define QUERY_CACHE_STATS_SEM	3
 #define PCP_REQUEST_SEM			4
 #define ACCEPT_FD_SEM			5
-#define MAIN_EXIT_HANDLER_SEM	6	/* used in exit_hander in pgpool main process */
+#define FOLLOW_PRIMARY_SEM		6
 #define MAX_REQUEST_QUEUE_SIZE	10
 
 #define MAX_SEC_WAIT_FOR_CLUSTER_TRANSATION 10	/* time in seconds to keep
@@ -479,6 +525,9 @@ typedef struct
 	int			conn_counter;
 	bool		switching;		/* it true, failover or failback is in
 								 * progress */
+	/* false if follow primary command or detach_false_primary in
+	 * execution */
+	bool		follow_primary_count;
 }			POOL_REQUEST_INFO;
 
 /* description of row. corresponding to RowDescription message */
@@ -571,7 +620,6 @@ typedef struct
 }			PGVersion;
 
 extern ProcessState processState;
-extern bool	reset_query_error;	/* true if error occurs in reset queries */
 
 extern POOL_CONNECTION_POOL * pool_connection_pool; /* connection pool */
 extern volatile sig_atomic_t backend_timer_expired; /* flag for connection
@@ -645,17 +693,12 @@ extern bool degenerate_backend_set_ex(int *node_id_set, int count, unsigned char
 extern bool promote_backend(int node_id, unsigned char flags);
 extern bool send_failback_request(int node_id, bool throw_error, unsigned char flags);
 
-
-extern void pool_set_timeout(int timeoutval);
-extern int	pool_check_fd(POOL_CONNECTION * cp);
-
 extern void pool_send_frontend_exits(POOL_CONNECTION_POOL * backend);
 
 extern int	pool_read_message_length(POOL_CONNECTION_POOL * cp);
 extern int *pool_read_message_length2(POOL_CONNECTION_POOL * cp);
 extern signed char pool_read_kind(POOL_CONNECTION_POOL * cp);
 extern int	pool_read_int(POOL_CONNECTION_POOL * cp);
-extern void pool_emit_log_for_message_length_diff(int *length_array, char *name);
 
 extern POOL_STATUS SimpleForwardToFrontend(char kind, POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend);
 extern POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, int len, char *contents);
@@ -760,7 +803,7 @@ extern PGVersion *Pgversion(POOL_CONNECTION_POOL * backend);
 extern void reset_variables(void);
 extern void reset_connection(void);
 extern void per_node_statement_log(POOL_CONNECTION_POOL * backend, int node_id, char *query);
-extern char per_node_error_log(POOL_CONNECTION_POOL * backend, int node_id, char *query, char *prefix, bool unread);
+extern void per_node_error_log(POOL_CONNECTION_POOL * backend, int node_id, char *query, char *prefix, bool unread);
 extern int	pool_extract_error_message(bool read_kind, POOL_CONNECTION * backend, int major, bool unread, char **message);
 extern POOL_STATUS do_command(POOL_CONNECTION * frontend, POOL_CONNECTION * backend,
 							  char *query, int protoMajor, int pid, int key, int no_ready_for_query);
@@ -828,5 +871,7 @@ extern POOL_NODE_STATUS * verify_backend_node_status(POOL_CONNECTION_POOL_SLOT *
 extern POOL_NODE_STATUS * pool_get_node_status(void);
 extern void pool_set_backend_status_changed_time(int backend_id);
 extern int	get_next_master_node(void);
+extern bool pool_acquire_follow_primary_lock(bool block);
+extern void pool_release_follow_primary_lock(void);
 
 #endif							/* POOL_H */
