@@ -52,7 +52,7 @@
 											 * lifecheck process should waits
 											 * before giving up while fetching
 											 * the configured watchdog node
-											 * infromation from watchdog
+											 * information from watchdog
 											 * process through IPC channel */
 
 /*
@@ -315,7 +315,8 @@ lifecheck_exit_handler(int sig)
 
 		if (wpid == -1 && errno != ECHILD)
 			ereport(WARNING,
-					(errmsg("wait() on lifecheck children failed. reason:%s", strerror(errno))));
+					(errmsg("wait() on lifecheck children failed"),
+					 errdetail("%m")));
 
 		if (g_hb_receiver_pid)
 			pfree(g_hb_receiver_pid);
@@ -358,14 +359,15 @@ fork_lifecheck_child(void)
 		/* Set the process type variable */
 		processType = PT_LIFECHECK;
 
-		/* call PCP child main */
+		/* call lifecheck child main */
 		POOL_SETMASK(&UnBlockSig);
 		lifecheck_main();
 	}
 	else if (pid == -1)
 	{
 		ereport(FATAL,
-				(errmsg("fork() failed. reason: %s", strerror(errno))));
+				(errmsg("fork() failed"),
+				 errdetail("%m")));
 	}
 
 	return pid;
@@ -825,6 +827,13 @@ check_pgpool_status_by_query(void)
 		thread_arg[i].lifeCheckNode = node;
 		thread_arg[i].password = password;
 		rc = watchdog_thread_create(&thread[i], &attr, thread_ping_pgpool, (void *) &thread_arg[i]);
+		if (rc)
+		{
+			ereport(WARNING,
+					(errmsg("failed to create thread for checking pgpool status by query for  %d (%s:%d)",
+							i, node->hostName, node->pgpoolPort),
+					 errdetail("pthread_create failed with error code %d: %s",rc, strerror(rc))));
+		}
 	}
 
 	pthread_attr_destroy(&attr);
@@ -1133,7 +1142,7 @@ wd_ping_all_server(void)
 				continue;
 			ereport(WARNING,
 					(errmsg("failed to check the ping status of trusted servers"),
-					 errdetail("waitpid failed with reason: %s", strerror(errno))));
+					 errdetail("waitpid failed with reason: %m")));
 			break;
 		}
 	}
